@@ -129,7 +129,7 @@ If nil, only read articles will be expired."
   "Load FILE and do a `read' there."
   (nnheader-temp-write nil
     (ignore-errors
-      (insert-file-contents file)
+      (nnheader-insert-file-contents file)
       (goto-char (point-min))
       (read (current-buffer)))))
 
@@ -539,7 +539,7 @@ the actual number of articles toggled is returned."
       (gnus-make-directory (file-name-directory file))
       (nnheader-temp-write file
 	(when (file-exists-p file)
-	  (insert-file-contents file))
+	  (nnheader-insert-file-contents file))
 	(goto-char (point-min))
 	(if nntp-server-list-active-group
 	    (progn
@@ -728,7 +728,7 @@ the actual number of articles toggled is returned."
 	(when (= (point-max) (point-min))
 	  (push (cons group (current-buffer)) gnus-agent-buffer-alist)
 	  (ignore-errors
-	    (insert-file-contents
+	    (nnheader-insert-file-contents
 	     (gnus-agent-article-name ".overview" group))))
 	(nnheader-find-nov-line (string-to-number (cdar crosses)))
 	(insert (string-to-number (cdar crosses)))
@@ -795,7 +795,7 @@ the actual number of articles toggled is returned."
   (goto-char (point-min))
   (set-buffer nntp-server-buffer)
   (erase-buffer)
-  (insert-file-contents file)
+  (nnheader-insert-file-contents file)
   (goto-char (point-max))
   (if (or (= (point-min) (point-max))
 	  (progn
@@ -1307,7 +1307,7 @@ The following commands are available:
 	     (set-buffer overview)
 	     (erase-buffer)
 	     (when (file-exists-p nov-file)
-	       (insert-file-contents nov-file))
+	       (nnheader-insert-file-contents nov-file))
 	     (goto-char (point-min))
 	     (setq article 0)
 	     (while (setq elem (pop articles))
